import {AppState, INIT_STATE, rootReducer} from '../../../shared/redux';
import {NgRedux} from '@angular-redux/store';
import {Inject, NgModule, NgZone, Optional} from '@angular/core';
import {RootStore} from '@angular-redux/store/lib/src/components/root-store';
import {async, inject, TestBed} from '@angular/core/testing';
import {HttpClient, HttpClientModule} from '@angular/common/http';
import {HttpClientTestingModule, HttpTestingController} from '@angular/common/http/testing';
import {createEpicMiddleware, Epic} from 'redux-observable';
import {EpicProvider, REDUX_EPIC} from '../../../shared/redux/provider';
import {Action} from 'redux';
import {einstiegspunkte} from '../../../shared/einstiegspunkte';

@NgModule({
  providers: [
    {
      provide: NgRedux,
      useFactory: (zone: NgZone) => new RootStore(zone),
      deps: [NgZone]
    },
  ]
})
export class ${Service_name}ReduxStoreModule {
  constructor(private reduxStore: NgRedux<AppState>,
              @Inject(REDUX_EPIC) @Optional() epics: Epic<Action, {}>[] = [],
              http: HttpClient) {

    const middleware = epics.map(epic => createEpicMiddleware(epic));
    this.reduxStore.configureStore(rootReducer, INIT_STATE, middleware);
  }
}

describe('${Service_name}StateService', () => {

  beforeEach(async(() => {
    TestBed.configureTestingModule({
      imports: [${Service_name}ReduxStoreModule, HttpClientModule, HttpClientTestingModule],
      providers: [
        ${Service_name}StateService,
        ...einstiegspunkte,
        EpicProvider(${Epic_name}Epic, [HttpClient, '${URL_alias}'])
      ]
    });
  }));

  describe('${Epic_name}Epic ', () => {

    it('in Anforderung', inject([NgRedux, ${Service_name}StateService],
      (store: NgRedux<AppState>, service: ${Service_name}StateService) => {

      // service call
      // service.foo();
  
      // check state
      // expect(store.getState().bar).toEqual('foo');
    }));

    it('im Erfolgsfall', inject([NgRedux, ${Service_name}StateService, HttpTestingController],
      (store: NgRedux<AppState>, service: ${Service_name}StateService, testingController: HttpTestingController) => {

      // service call
      // service.foo();
      
      const request = testingController.expectOne('/rueckgriff/api/path/');
      request.flush(/*MOCK_OBJECT*/);
  
      // check state
      // expect(store.getState().bar).toEqual('foo');
    }));

    it('im Fehlerfall', inject([NgRedux, ${Service_name}StateService, HttpTestingController],
      (store: NgRedux<AppState>, service: ${Service_name}StateService, testingController: HttpTestingController) => {

        // service call
        // service.foo();
        
        const mockErrorResponse = {
          status: 404, statusText: 'Bad Request'
        };

        const data = 'Invalid request';
        
        const request = testingController.expectOne('/rueckgriff/api/path/');
        request.flush(data, mockErrorResponse);
        
        // check state
        // expect(store.getState().bar).toEqual('foo');
    }));
  });
});
