/* tslint:disable no-any */
import {BrowserModule} from '@angular/platform-browser';
import {Inject, NgModule, Optional} from '@angular/core';
import {AppComponent} from './app.component';
import {NgRedux, NgReduxModule} from '@angular-redux/store';
import {AppState, rootReducer} from './shared/redux';
import {HttpClientModule} from '@angular/common/http';
import {CoreModule} from './core/core.module';
import {createEpicMiddleware, Epic} from 'redux-observable';
import {REDUX_EPIC} from './shared/redux/provider';
import {Action, applyMiddleware, compose, createStore, Store} from 'redux';
import 'hammerjs';
import {TimelineModule} from './modules/timeline/timeline.module';
import {TimelineStateService} from './core/services/timeline/timeline.service';

@NgModule({
  bootstrap: [AppComponent],
  declarations: [
    AppComponent
  ],
  entryComponents: [],
  imports: [
    CoreModule,
    BrowserModule,
    NgReduxModule,
    TimelineModule,
    HttpClientModule
  ],
  providers: [
    TimelineStateService
  ]
})
export class AppModule {
  constructor(
    private reduxStore: NgRedux<AppState>,
    @Inject(REDUX_EPIC) @Optional() epics: Epic<Action>[] = []) {

    const epicMiddleware = createEpicMiddleware();

    const composeEnhancers = (<any>window).__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ || compose;

    const store: Store<AppState> = createStore(
      rootReducer,
      composeEnhancers(
        applyMiddleware(epicMiddleware)
      )
    );

    for (const epic of epics) {
      epicMiddleware.run(epic);
    }

    this.reduxStore.provideStore(store);
  }
}
