/* tslint:disable:no-any */
import {concat, Observable, of as observableOf, throwError} from 'rxjs';
import {Action} from '../../../shared/redux';
import {TemplateUrl} from '../../../shared/templateurl';
import {HttpClient} from '@angular/common/http';
import {createAction} from '../../../shared/redux/actions';
import {catchError, delay, map, retryWhen, switchMap, take} from 'rxjs/operators';
import {ActionsObservable, Epic} from 'redux-observable';
import {IEvent} from '../../../shared/models/event.model';
import {HTTP_REQUEST_DELAY, HTTP_REQUEST_TRIES} from '../../../configs/timings.config';

export interface TimelineState {
  loadTimelineState: string;
  loadTimelineError: string;
  events: IEvent[];
}

export const INIT_STATE_TIMELINE_LADEN: TimelineState = {
  loadTimelineState: null,
  loadTimelineError: null,
  events: null
};

export const LOAD_TIMELINE_REQUEST = 'LOAD_TIMELINE_REQUEST';
export const LOAD_TIMELINE_SUCCESS = 'LOAD_TIMELINE_SUCCESS';
export const LOAD_TIMELINE_ERROR = 'LOAD_TIMELINE_ERROR';

export enum States {
  Request = 'Request',
  Success = 'Success',
  Error = 'Error'
}

export function timelineReducer(
  state: TimelineState = INIT_STATE_TIMELINE_LADEN, action: Action<any>): TimelineState {
  switch (action.type) {
    case LOAD_TIMELINE_REQUEST : {
      return {
        ...state,
        loadTimelineState: States.Request
      };
    }
    case LOAD_TIMELINE_SUCCESS : {
      return {
        ...state,
        events: action.payload,
        loadTimelineState: States.Success
      };
    }
    case LOAD_TIMELINE_ERROR :
      return {
        ...state,
        events: null,
        loadTimelineError: action.payload,
        loadTimelineState: States.Error
      };
    default:
      return state;
  }
}

export function loadTimelineEpic(http: HttpClient, url: TemplateUrl): Epic<Action<any>> {
  const FEHLERAUSGABE = 'Fehler beim Laden der Timeline';

  return (action$: ActionsObservable<any>) => action$
    .ofType(LOAD_TIMELINE_REQUEST)
    .pipe(
      switchMap((action: Action<any>) => {
        const vorgangId = action.payload;

        return lade(http, url, vorgangId)
          .pipe(
            map((events: IEvent[]) => {
              return createAction(LOAD_TIMELINE_SUCCESS, events);
            }),
            retryWhen(errors =>
                concat(
                    errors.pipe(
                      delay(HTTP_REQUEST_DELAY),
                      take(HTTP_REQUEST_TRIES)
                    ),
                    throwError(errors)
                )
            ),
            catchError(() => {
              console.log(FEHLERAUSGABE);

              return observableOf(createAction(LOAD_TIMELINE_ERROR, FEHLERAUSGABE));
            })
          );
      })
    );
}

function lade(http: HttpClient, url: TemplateUrl, processId: string): Observable<IEvent[]> {
  return http.get<IEvent[]>(url({processId: processId}));
}
