import {inject, TestBed} from '@angular/core/testing';
import {NgRedux} from '@angular-redux/store';
import {Inject, NgModule, NgZone, Optional} from '@angular/core';
import {RootStore} from '@angular-redux/store/lib/src/components/root-store';
import {HttpClient, HttpClientModule} from '@angular/common/http';
import {HttpClientTestingModule, HttpTestingController} from '@angular/common/http/testing';
import {Action, applyMiddleware, createStore, Store} from 'redux';
import {createEpicMiddleware, Epic} from 'redux-observable';
import {loadTimelineEpic} from './timeline.redux';
import {TIMELINE} from '../../mocks/timeline.mock';
import {AppState, rootReducer} from '../../../shared/redux';
import {EpicProvider, REDUX_EPIC} from '../../../shared/redux/provider';
import {TimelineStateService} from './timeline.service';
import {einstiegspunkte} from '../../../shared/einstiegspunkte';
import {Router} from '@angular/router';
import {UUID_VORGANG_ID} from '../../mocks/uuids.mock';

@NgModule({
  providers: [
    {
      deps: [NgZone],
      provide: NgRedux,
      useFactory: (zone: NgZone) => new RootStore(zone),
    },
  ]
})
export class TimelineReduxStoreModule {
  constructor(private reduxStore: NgRedux<AppState>,
              @Inject(REDUX_EPIC) @Optional() epics: Epic<Action>[] = []) {

    const epicMiddleware = createEpicMiddleware();

    const store: Store<AppState> = createStore(rootReducer, applyMiddleware(epicMiddleware));

    for (const epic of epics) {
      epicMiddleware.run(epic);
    }

    this.reduxStore.provideStore(store);
  }
}

describe('TimelineStateService - Laden der Timeline', () => {

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [TimelineReduxStoreModule, HttpClientModule, HttpClientTestingModule],
      providers: [
        {
          provide: Router,
          useValue: {
            ...jasmine.createSpyObj('Router', ['navigate'])
          }
        },
        TimelineStateService,
        ...einstiegspunkte,
        EpicProvider(loadTimelineEpic, [HttpClient, 'events.laden', Router])
      ]
    });
  });

  describe('loadTimelineEpic ', () => {

    it('im Anforderung', inject([NgRedux, TimelineStateService, HttpTestingController],
      (store: NgRedux<AppState>, service: TimelineStateService) => {

        service.loadTimeline(UUID_VORGANG_ID);

        expect(store.getState().timeline).toEqual({
          events: undefined,
          loadTimelineError: '',
          loadTimelineState: 'LADE'
        });
      }));

    it('im Erfolgsfall', inject([NgRedux, TimelineStateService, HttpTestingController],
      (store: NgRedux<AppState>, service: TimelineStateService, testingController: HttpTestingController) => {

        service.loadTimeline(UUID_VORGANG_ID);
        const request = testingController.expectOne('/rueckgriff/api/rueckgriffe/' + UUID_VORGANG_ID + '/timeline');
        request.flush(TIMELINE);

        expect(store.getState().timeline).toEqual({
          events: TIMELINE,
          loadTimelineError: '',
          loadTimelineState: 'GELADEN',
        });
      }));

    it('im Fehlerfall', inject([NgRedux, TimelineStateService, HttpTestingController],
      (store: NgRedux<AppState>, service: TimelineStateService, testingController: HttpTestingController) => {

        service.loadTimeline(UUID_VORGANG_ID);

        const request = testingController.expectOne('/rueckgriff/api/rueckgriffe/' + UUID_VORGANG_ID + '/timeline');
        request.flush([], {status: 404, statusText: 'Fehler beim Laden der Timeline'});

        expect(store.getState().timeline).toEqual({
          events: undefined,
          loadTimelineError: 'Fehler beim Laden der Timeline',
          loadTimelineState: 'FEHLER'
        });
      }));
  });
});
